﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xaml;
using GME.Common.UI.DragDropFramework;
using GME.Common.UI.Helpers;
using Infragistics.Windows.Commands;
using Infragistics.Windows.Controls;
using Infragistics.Windows.DataPresenter;
using Utilities = Infragistics.Windows.Utilities;

namespace GridTester
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public IEnumerable<string> ListItems { get; set; }

        public string ButtonText { get { return "MyText"; } }

        public MainWindow()
        {
            this.DataContext = this;

            InitializeComponent();

            /*for (int i = 1; i <= 50; i++)
            {
                xamDataGrid1.DefaultFieldLayout.Fields.Add
                (new Field
                {
                    Name = "Field" + i,
                    Label = "Field" + i,
                });
            }

            xamDataGrid1.DefaultFieldLayout.Fields.Add(new Field
            {
                Name = "SortCol",
                Visibility = Visibility.Visible,
                DataType = typeof(int),
                FixedLocation = FixedFieldLocation.FixedToFarEdge,
                Settings = { AllowHiding = AllowFieldHiding.Never, AllowFixing = AllowFieldFixing.No }
            });*/
        }

        public IBindingList ViewModels { get; protected set; }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            const int defaultRowCount = 40;
            var dealList = new BindingList<TestViewModel>();
            ViewModels = dealList;

            for (var count = 0; count < defaultRowCount; count++)
            {
                var vm = new TestViewModel { Field1 = 11.1175m };
                dealList.Add(vm);
            }

            xamDataGrid1.DataSource = ViewModels;
        }

        private void XamDataGrid1Sorting(object sender, Infragistics.Windows.DataPresenter.Events.SortingEventArgs e)
        {
            if (e.SortDescription != null && e.SortDescription.Direction == ListSortDirection.Ascending)
            {
                e.Cancel = true;

                e.FieldLayout.SortedFields.Clear();

                e.FieldLayout.SortedFields.Add(new FieldSortDescription("SortCol", e.SortDescription.Direction,
                                                                        false));

                e.FieldLayout.SortedFields.Add(new FieldSortDescription(e.SortDescription.FieldName,
                                                                        e.SortDescription.Direction,
                                                                        e.SortDescription.IsGroupBy));
            }
        }

        private void xamDataGrid1_CellUpdated(object sender, Infragistics.Windows.DataPresenter.Events.CellUpdatedEventArgs e)
        {
            var testViewModel = e.Cell.Record.DataItem as TestViewModel;

            if (testViewModel != null && testViewModel.SortCol != int.MinValue)
                testViewModel.SortCol = int.MinValue;
        }

        #region Field Chooser

        // NOTE: 
        // In next iteration, Drag/Drop actions would be refactored by using the Drag/Drop framework
        // once the underlying issue (using FieldChooser as source to drag field chooser entry) is investigated/resolved.

        #region Field and Properties

        private bool _isDragging;
        private FrameworkElement _dragScope;
        private bool _canInitiateDrag;
        private bool _dragHasLeftScope = false;
        private Point _startPoint;
        private DefaultAdorner _adorner = null;

        #endregion Field and Properties

        #region Event Handlers

        private void OnFieldChooserOpening(object sender, Infragistics.Windows.DataPresenter.Events.FieldChooserOpeningEventArgs e)
        {
            var fieldChooser = e.FieldChooser;
            fieldChooser.FieldGroupSelectorVisibility = Visibility.Collapsed;
            fieldChooser.FieldDisplayOrder = FieldChooserDisplayOrder.SameAsDataPresenter;

            fieldChooser.AllowDrop = true;
            fieldChooser.Drop += FieldChooserDrop;
            fieldChooser.PreviewMouseMove += FieldChooserPreviewMouseMove;
            fieldChooser.PreviewMouseLeftButtonDown += FieldChooserPreviewMouseLeftButtonDown;
            fieldChooser.Unloaded += FieldChooserUnloaded;
        }

        void FieldChooserUnloaded(object sender, RoutedEventArgs e)
        {
            var fieldChooser = sender as FieldChooser;
            if (fieldChooser != null)
            {
                fieldChooser.Drop -= FieldChooserDrop;
                fieldChooser.PreviewMouseMove -= FieldChooserPreviewMouseMove;
                fieldChooser.PreviewMouseLeftButtonDown -= FieldChooserPreviewMouseLeftButtonDown;
                fieldChooser.Unloaded -= FieldChooserUnloaded;
            }
        }

        void FieldChooserPreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            var source = e.OriginalSource as DependencyObject;
            if (source == null || Infragistics.Windows.Utilities.GetAncestorFromType(source, typeof(ListBoxItem), true) == null)
            {
                _canInitiateDrag = false;
            }
            else
            {
                    // Get Start position for dragging
                _startPoint = e.GetPosition(null);
                _canInitiateDrag = true;                
            }
        }

        void FieldChooserPreviewMouseMove(object sender, MouseEventArgs e)
        {
            // Ensure that the user does not drag by accident
            if (e.LeftButton == MouseButtonState.Pressed && !_isDragging && _canInitiateDrag)
            {
                Point position = e.GetPosition(null);

                if ((Math.Abs(position.X - _startPoint.X) > SystemParameters.MinimumHorizontalDragDistance ||
                    Math.Abs(position.Y - _startPoint.Y) > SystemParameters.MinimumVerticalDragDistance) && Keyboard.IsKeyDown(Key.LeftShift))
                {
                    StartDragInProcAdorner(e);
                }
            }
        }

        void DragScopeDragLeave(object sender, DragEventArgs e)
        {
            if (e.OriginalSource == _dragScope)
            {
                var p = e.GetPosition(_dragScope);
                var r = VisualTreeHelper.GetContentBounds(_dragScope);
                if (!r.Contains(p))
                {
                    _dragHasLeftScope = true;
                    e.Handled = true;
                }
            }
        }

        void DragScopeDragOver(object sender, DragEventArgs args)
        {
            if (_adorner != null)
            {
                _adorner.SetMousePosition(args.GetPosition(_dragScope));
            }
        }

        void DragScopeQueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            if (_dragHasLeftScope)
            {
                e.Action = DragAction.Cancel;
                e.Handled = true;
            }
        }

        private void FieldChooserDrop(object sender, DragEventArgs e)
        {
            var fieldChooser = sender as FieldChooser;

            if (fieldChooser != null)
            {
                //Get Instance of the dragger DataRecordPresenter and DataRecord
                var dragEntry = e.Data.GetData(typeof(FieldChooserEntry)) as FieldChooserEntry;

                if (dragEntry != null)
                {
                    #region Via ActualPosition
                    {
                        var fields = dragEntry.Field.Owner.Fields;
                        var dropEntry =
                            GetObjectDataFromPoint(fieldChooser, e.GetPosition(fieldChooser)) as FieldChooserEntry ??
                            fieldChooser.CurrentFields[0];

                        var dragEntryPos = dragEntry.Field.ActualPosition;

                        fields[dragEntry.Field.Index].ActualPosition
                            = new FieldPosition(dropEntry.Field.ActualPosition.Column, dragEntryPos.Row,
                                                dragEntryPos.ColumnSpan, dragEntryPos.RowSpan);

                        dragEntry.Field.Owner.EnsureUniqueFieldPositions();

                        fieldChooser.DataPresenter.UpdateLayout();
                        fieldChooser.ResetFieldDisplayOrder();
                        fieldChooser.FieldDisplayOrder = FieldChooserDisplayOrder.SameAsDataPresenter;
                    }

                    #endregion
                }
            }
        }

        #endregion Event Handlers

        #region Helper functions

        /// <summary>
        /// Displaying the Drag process in the Adorner Layer
        /// </summary>
        /// <param name="e"></param>
        private void StartDragInProcAdorner(MouseEventArgs e)
        {
            var fieldChooser = e.Source as FieldChooser;

            if (fieldChooser != null)
            {
                var dragEntry = fieldChooser.SelectedField;

                if (dragEntry == null)
                    return;

                _dragScope = fieldChooser as FrameworkElement;

                var previousDrop = _dragScope.AllowDrop;
                _dragScope.AllowDrop = true;

                var draghandler = new DragEventHandler(DragScopeDragOver);
                _dragScope.PreviewDragOver += draghandler;

                var dragleavehandler = new DragEventHandler(DragScopeDragLeave);
                _dragScope.DragLeave += dragleavehandler;

                var queryhandler = new QueryContinueDragEventHandler(DragScopeQueryContinueDrag);
                _dragScope.QueryContinueDrag += queryhandler;

                try
                {
                    var listBoxes = new List<ListBox>();
                    VisualTreeHelperHelper.GetChildren((DependencyObject)fieldChooser, listBoxes);

                    var listItem = listBoxes[0].ItemContainerGenerator.ContainerFromItem(dragEntry);
                    var dragElement =
                        Infragistics.Windows.Utilities.GetDescendantFromType(listItem, typeof(LabelPresenter), true) as LabelPresenter;

                    _adorner = new DefaultAdorner(_dragScope, dragElement,new Point(30,10) , 0.75);
                    AdornerLayer.GetAdornerLayer(_dragScope).Add(_adorner);

                    _isDragging = true;
                    _dragHasLeftScope = false;

                    var data = new DataObject(dragEntry);
                    System.Windows.DragDrop.DoDragDrop(fieldChooser, data, DragDropEffects.Move);
                }
                catch (Exception ex)
                {
                    Trace.TraceError("Unable to perform drag/drop on FieldChooser {0}", ex);
                }
                finally
                {
                    _dragScope.AllowDrop = previousDrop;

                    if (_adorner != null)
                    {
                        AdornerLayer.GetAdornerLayer(_dragScope).Remove(_adorner);
                        _adorner = null;
                    }

                    _dragScope.DragLeave -= dragleavehandler;
                    _dragScope.QueryContinueDrag -= queryhandler;
                    _dragScope.PreviewDragOver -= draghandler;

                    _isDragging = false;
                }
            }
        }

        private static object GetObjectDataFromPoint(FieldChooser fieldChooser, Point point)
        {
            var hitTest = VisualTreeHelper.HitTest(fieldChooser, point);

            if (hitTest.VisualHit != null)
            {
                var element = hitTest.VisualHit;

                if ((element is FrameworkElement) && ((FrameworkElement)element).DataContext is FieldChooserEntry)
                    return ((FrameworkElement)element).DataContext;
                else
                {
                    var listItem = VisualTreeHelperHelper.GetParent<ListBoxItem>(element);

                    if (listItem != null)
                        return listItem.DataContext;
                }
            }

            return null;
        }

        #endregion

        #endregion Field Chooser
    }

    public class SortComparer : IComparer
    {
        #region Implementation of IComparer

        /// <summary>
        /// Compares two objects and returns a value indicating whether one is less than, equal to, or greater than the other.
        /// </summary>
        /// <returns>
        /// A signed integer that indicates the relative values of <paramref name="x"/> and <paramref name="y"/>, as shown in the following table.Value Meaning Less than zero <paramref name="x"/> is less than <paramref name="y"/>. Zero <paramref name="x"/> equals <paramref name="y"/>. Greater than zero <paramref name="x"/> is greater than <paramref name="y"/>. 
        /// </returns>
        /// <param name="x">The first object to compare. </param><param name="y">The second object to compare. </param><exception cref="T:System.ArgumentException">Neither <paramref name="x"/> nor <paramref name="y"/> implements the <see cref="T:System.IComparable"/> interface.-or- <paramref name="x"/> and <paramref name="y"/> are of different types and neither one can handle comparisons with the other. </exception><filterpriority>2</filterpriority>
        public int Compare(object x, object y)
        {
            if (x != null && y != null)
                return Comparer.DefaultInvariant.Compare(x, y);
            else if (x == null && y == null)
                return int.MinValue;
            else
                return (x == null ? y.GetHashCode() : x.GetHashCode());
        }

        #endregion
    }


    [MarkupExtensionReturnTypeAttribute(typeof(Object))]
    [TypeConverterAttribute(typeof(DynamicResourceExtensionConverter))]
    public class GetValueExtension : MarkupExtension
    {
        public static readonly DependencyProperty GetValueProperty =
            DependencyProperty.RegisterAttached("GetValue",
                                                                       typeof(string), typeof(GetValueExtension));

        public static void SetGetValue(DependencyObject o, string value)
        {
            o.SetValue(GetValueProperty, value);
        }

        public static string GetGetValue(DependencyObject o)
        {
            return o.GetValue(GetValueProperty) as string;
        }

        private string Path { get; set; }

        public GetValueExtension(string path)
        {
            Path = path;
        }

        #region Overrides of MarkupExtension

        /// <summary>
        /// When implemented in a derived class, returns an object that is set as the value of the target property for this markup extension. 
        /// </summary>
        /// <returns>
        /// The object value to set on the property where the extension is applied. 
        /// </returns>
        /// <param name="serviceProvider">Object that can provide services for the markup extension.</param>
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            var binding = new Binding(Path);
            var target = serviceProvider.GetService(typeof(IProvideValueTarget)) as IProvideValueTarget;

            if (target != null)
            {
                var element = target.TargetObject as FrameworkElement;

                if (element == null)
                {
                    var root = serviceProvider.GetService(typeof(IRootObjectProvider)) as IRootObjectProvider;

                    if (root != null)
                        element = root.RootObject as FrameworkElement;
                }

                if (element != null)
                {
                    var exp = element.SetBinding(GetValueProperty, binding);
                    exp.UpdateTarget();
                    var result = element.GetValue(GetValueProperty);
                    return result + " MODIFIED";
                }
            }

            return null;
        }

        #endregion
    }
}
